%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%              TRABAJO FIN DE MASTER              %%%%%%%%%%%%%
%
%     DESARROLLO DE UNA HERRAMIENTA DE DIAGNOSTICO DE FALLOS EN MOTORES
%                   DE INDUCCION MEDIANTE LA TECNICA ADABOOST
%
%                           JUAN OBREGN SANDOVAL
%%%%%%%%%%%%%                                                 %%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%-------------------------------------------------------------------------%

%   ENTRENAMIENTO Y ENSAYO DE CLASIFICADORES BINARIOS MEDIANTE ADABOOST

%-------------------------------------------------------------------------%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear variables;
clc;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                     INICIALIZACION DE VARIABLES
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Parametros auxiliares
Num_caract=7;                   % Longitud de busqueda de strings para
                                % elegir datos acordes a Alimentacion y a
                                % Nivel_Carga
                                
eliminar=[1 2 3 4 5 6 9 10]';   % Filas a eliminar de Datos (datos no
                                % utilizados, no son estimadores ni clases)
                                
Reproduc=1;     % Si =1--> Se fija el valor de la semilla para 
                % reproductibilidad
Seed=1;         % Inicializacion semilla                                                               

% Parametros para particionado de datos
Partition=0.3;                  % Porcentaje datos usados como test 
                    
                                
% Parametros para seleccion de valores de medicion
Restric_Alim=1;     % Si =1--> Se trabajara con la alimentacion elegida
Alimentacion='RED'; % Se determina el tipo de alimentacion al motor elegida

Restric_Carga=1;    % Si =1--> Se trabajara con la carga elegida
Nivel_Carga=1;      % Se determina el nivel de carga de funcionamiento del motor elegida

Clase=5;            % Se determina la clase a estudiar elegida

% Parametros del clasificador
NLearn=50;                      % Numero de ciclos clasificador                                                       
KFold=20;                       % Subconjuntos para el CV del clasificador 

                            
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                     CARGA Y PREPARADO DE DATOS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Carga de valores del excel
[Datos_Inicio, Texto_Inicio, Todo_Inicio]=xlsread('datos_ignacio.xlsx'); % Cargamos valores del excel

Datos=Datos_Inicio;
Texto=Texto_Inicio;
Todo=Todo_Inicio;

% Tratamiento de los datos segun parametros elegidos previamente
Texto(1,:)=[];             % Eliminamos fila para cuadrar dimensiones con "Datos"
Todo(1,:)=[];              % Eliminamos fila para cuadrar dimensiones con "Datos"

if Restric_Alim==1
    a=find(strncmp(Alimentacion,Texto(:,2),Num_caract)); % Buscar valores correspondientes a alimentacion elegida
    Datos=Datos(a,:);                                    % Matriz con valores correspondientes a la alimentacion elegida
    Texto=Texto(a,:);                                    % Matriz con valores correspondientes a la alimentacion elegida
    Todo=Todo(a,:);                                      % Matriz con valores correspondientes a la alimentacion elegida
else
end

if Restric_Carga==1
    b=find(Datos(:,2)==Nivel_Carga); % Buscar valores numericos correspondientes a carga elegida
    Datos=Datos(b,:);       % Matriz con valores correspondientes a los de la carga elegida
    Texto=Texto(b,:); % Matriz con valores correspondientes a los de la carga elegida
    Todo=Todo(b,:);   % Matriz con valores correspondientes a los de la carga elegida
else
end

Datos(:,eliminar)=[];
Texto(:,eliminar+3)=[];
Todo(:,eliminar+3)=[];

% Busqueda de las medidas de cada clase
Clases=unique(Todo(:,3));
Num_Clases=length(Clases);        % Se determina el numero de clases existentes

R1=find(strncmp('R1',Todo(:,3),Num_caract));
R2=find(strncmp('R2',Todo(:,3),Num_caract));
R3=find(strncmp('R3',Todo(:,3),Num_caract));
R4=find(strncmp('R4',Todo(:,3),Num_caract));
R5=find(strncmp('R5',Todo(:,3),Num_caract));

X=Datos;
Y_Clases=Todo(:,3);
Long_Y=length(Y_Clases);

Y=ones([Long_Y,1]);
Y(R1)=1;
Y(R2)=2;
Y(R3)=3;
Y(R4)=4;
Y(R5)=5;

% Preparamos los datos de la muestra reducida
if Reproduc==1
    rng(Seed,'twister') % Para reproducibilidad de las mediciones
else
end

cvpart=cvpartition(Y,'holdout',Partition);

X_Training=X(cvpart.training,:);
X_Test=X(cvpart.test,:);

Y_Training=Y(cvpart.training,:);
Y_Test=Y(cvpart.test,:);

Long_X_Test=length(X_Test);
Y_Binario=zeros(Num_Clases,Long_Y);
for i=1:Num_Clases
    Y_Binario(i,:)=(Y==i);
end

% Preparamos los datos test para representar la matriz de confusion
Y_Test_Mat_Conf=zeros(Num_Clases,Long_X_Test);
for i=1:Num_Clases
    Y_Test_Mat_Conf(i,:)=(Y_Test==i);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                     ENTRENAMIENTO DEL CLASIFICADOR
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%            

Error_Clasif_AdaBoostM1=0;      % Variable para detectar cuando aplicar
                                % clasifiacion mediante arbol de decision

% Entrenamiento del clasificador
Y_Training_Clasif_Binario=(Y_Training==Clase);

Clasificador=fitensemble(X_Training,Y_Training_Clasif_Binario,'AdaBoostM1',NLearn,'Tree');

if isempty(Clasificador.TrainedWeights)
    Clasificador=fitctree(X_Training,Y_Training_Clasif_Binario);
    Clasificador_Aux=fitctree(X,Y_Binario(Clase,:));
    display('Error de clasificacion 0 Clasificador')
else
    Error_Clasif_AdaBoostM1=1;
    Clasificador_Aux=fitensemble(X,Y_Binario(Clase,:),'AdaBoostM1',NLearn,'Tree');
end


% Cross Validation  
Clasificador_CV=crossval(Clasificador_Aux,'KFold',KFold);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%               CLASIFICACIN DE LA PARTICION DE PRUEBA
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Clasificacion=predict(Clasificador,X_Test);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%     ERROR DE RESISTITUCION Y ERROR DE CLASIFICACION MEDIANTE CV
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if Error_Clasif_AdaBoostM1==1
    RsLoss=resubLoss(Clasificador,'Mode','Cumulative');
    GenError=kfoldLoss(Clasificador_CV,'Mode','Cumulative');
else
    RsLoss=resubLoss(Clasificador)*ones(1,NLearn)';
    GenError=kfoldLoss(Clasificador_CV)*ones(1,NLearn)';
end
RsLoss=RsLoss*100;
GenError=GenError*100;

    figure;
    plot((1:1:NLearn),RsLoss,(1:1:NLearn),GenError);
    title('\fontsize{24} Error del clasificador');
    legend('\fontsize{18} Resubstitution Classification Cost','\fontsize{18} Cross-Validation Loss');
    xlabel('\fontsize{20} Number of Learning Cycles');ylabel('\fontsize{20} Loss (%)');
    ylim([0 inf]);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%        
%    MATRIZ DE CONFUSION Y CURVA CARACTERISTICA OPERATIVA DEL RECEPTOR
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Representamos la matriz de confusion
figure
plotconfusion(Y_Test_Mat_Conf(Clase,:),Clasificacion')

% Representamos la curva caracteristica operativa del receptor
figure
plotroc(Y_Test_Mat_Conf(Clase,:),Clasificacion')