%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%              TRABAJO FIN DE MASTER              %%%%%%%%%%%%%
%
%     DESARROLLO DE UNA HERRAMIENTA DE DIAGNOSTICO DE FALLOS EN MOTORES
%                   DE INDUCCION MEDIANTE LA TECNICA ADABOOST
%
%                           JUAN OBREGN SANDOVAL
%%%%%%%%%%%%%                                                 %%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%-------------------------------------------------------------------------%

%  ENTRENAMIENTO Y ENSAYO DE METODOS DE CLASIFICACION MULTICLASE MEDIANTE
%   COMBINACION DE CLASIFICADORES BINARIOS Y MULTICLASE ADABOOST Y ARBOL

%-------------------------------------------------------------------------%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear variables;
clc;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                     INICIALIZACION DE VARIABLES
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Parametros auxiliares

Num_caract=7;                   % Longitud de busqueda de strings para
                                % elegir datos acordes a Alimentacion y a
                                % Nivel_Carga
                                
eliminar=[1 2 3 4 5 6 9 10]';   % Filas a eliminar de Datos (datos no
                                % utilizados, no son estimadores ni clases)
                                
Reproduc=1;     % Si =1--> Se fija el valor de la semilla para 
                % reproductibilidad
Seed=320;         % Inicializacion semilla                                                               

% Parametro para particionado de datos

Partition=0.3;                  % Porcentaje datos usados como test 
                    
                                
% Parametros para seleccion de valores de medicion

Restric_Alim=0;     % Si =0--> Se trabajara con todas las alimentaciones
Alimentacion='ABB1'; % Se determina el tipo de alimentacion al motor elegida

Restric_Carga=0;    % Si =0--> Se trabajara con todas las cargas
Nivel_Carga=2;      % Se determina el nivel de carga de funcionamiento del motor elegida

% Eleccion del metodo de clasificacion

Metodo=6;           % Si =1--> Cascada de clasificadores binarios
                    % Si =2--> Clasificador Clasificador multiclase
                    % Si =3--> Clasificador binario R1vsAll + Clasificador
                    % multiclase
                    % Si =4--> Clasificador binario R1vsAll + Clasificador
                    % binario R2vsResto + Clasificador multiclase
                    % Si =5--> Clasificadores binarios y clasificador 
                    % multiclase utilizados simultaneamente
                    % Si =6--> Clasificadores binarios y clasificador 
                    % multiclase utilizados simultaneamente, con preponderancia
                    % a la clasificacin como 1

Muestra_Reducida=0;             % Si =1--> Clasificadores entrenan con los
                                % datos correspondientes reducidos
                                
% Parametros de los clasificadores

NLearn_R1=50;             % Numero de ciclos clasificador para R1
NLearn_R2=50;             % Numero de ciclos clasificador para R2
NLearn_R3=50;             % Numero de ciclos clasificador para R3
NLearn_R4=50;             % Numero de ciclos clasificador para R4
NLearn_R5=50;             % Numero de ciclos clasificador para R5

NLearn_Multi=50;          % Numero de ciclos clasificador multiclase

KFold=20;                       % Subconjuntos para el CV del clasificador 


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                       CARGA Y PREPARADO DE DATOS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Carga de valores del excel

[Datos_Inicio, Texto_Inicio, Todo_Inicio]=xlsread('datos_ignacio.xlsx'); % Cargamos valores del excel

Datos=Datos_Inicio;
Texto=Texto_Inicio;
Todo=Todo_Inicio;

% Tratamiento de los datos segun parametros elegidos previamente

Texto(1,:)=[];      % Eliminamos fila para cuadrar dimensiones con "Datos"
Todo(1,:)=[];       % Eliminamos fila para cuadrar dimensiones con "Datos"

if Restric_Alim==1
    a=find(strncmp(Alimentacion,Texto(:,2),Num_caract)); % Buscar valores correspondientes a alimentacion elegida
    Datos=Datos(a,:);       % Matriz con valores correspondientes a la alimentacion elegida
    Texto=Texto(a,:); % Matriz con valores correspondientes a la alimentacion elegida
    Todo=Todo(a,:);   % Matriz con valores correspondientes a la alimentacion elegida
else
end

if Restric_Carga==1
    b=find(Datos(:,2)==Nivel_Carga); % Buscar valores numericos correspondientes a carga elegida
    Datos=Datos(b,:);       % Matriz con valores correspondientes a los de la carga elegida
    Texto=Texto(b,:); % Matriz con valores correspondientes a los de la carga elegida
    Todo=Todo(b,:);   % Matriz con valores correspondientes a los de la carga elegida
else
end

Datos(:,eliminar)=[];
Texto(:,eliminar+3)=[];%Sobra
Todo(:,eliminar+3)=[];%Sobra

% Busqueda de las medidas de cada clase

Clases=unique(Todo(:,3));
Num_Clases=length(Clases);        % Se determina el numero de clases existentes

R1=find(strncmp('R1',Todo(:,3),Num_caract));
R2=find(strncmp('R2',Todo(:,3),Num_caract));
R3=find(strncmp('R3',Todo(:,3),Num_caract));
R4=find(strncmp('R4',Todo(:,3),Num_caract));
R5=find(strncmp('R5',Todo(:,3),Num_caract));

X=Datos;
Y_Clases=Todo(:,3);
Long_Y=length(Y_Clases);

Y=ones([Long_Y,1]);
Y(R1)=1;
Y(R2)=2;
Y(R3)=3;
Y(R4)=4;
Y(R5)=5;

% Preparamos los datos de la muestra reducida

if Reproduc==1
    rng(Seed,'twister') % Para reproducibilidad de las mediciones
else
end

cvpart=cvpartition(Y,'holdout',Partition);

X_Training=X(cvpart.training,:);
X_Test=X(cvpart.test,:);

Y_Training=Y(cvpart.training,:);
Y_Test=Y(cvpart.test,:);

Long_X_Test=length(X_Test);

X_Training_Clasif_R2vsR3R4R5=X_Training(Y_Training>1,:);
X_Training_Clasif_R3vsR4R5=X_Training(Y_Training>2,:);
X_Training_Clasif_R4vsR5=X_Training(Y_Training>3,:);

Y_Training_Clasif_R2vsR3R4R5=Y_Training(Y_Training>1);
Y_Training_Clasif_R3vsR4R5=Y_Training(Y_Training>2);
Y_Training_Clasif_R4vsR5=Y_Training(Y_Training>3);

Y_Binario=zeros(Num_Clases,Long_Y);
for i=1:Num_Clases
    Y_Binario(i,:)=(Y==i);
end

% Preparamos los datos test para representar la matriz de confusion
Y_Test_Mat_Conf=zeros(Num_Clases,Long_X_Test);
for i=1:Num_Clases
    Y_Test_Mat_Conf(i,:)=(Y_Test==i);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%               ENTRENAMIENTO DE CLASIFICADORES Y PREDICCION
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Error_Clasif_AdaboostM1=zeros(1,Num_Clases);

if Metodo==1
% Cascada de clasificadores binarios (uno vs restantes)
   
    % Clasificador R1vsAll
    Y_Training_Clasif_Binario_R1=(Y_Training==1);
    Clasificador_R1=fitensemble(X_Training,Y_Training_Clasif_Binario_R1,'AdaBoostM1',NLearn_R1,'Tree');
    if isempty(Clasificador_R1.TrainedWeights)
        Clasificador_R1=fitctree(X_Training,Y_Training_Clasif_Binario_R1);
        Clasificador_Aux_R1=fitctree(X,Y_Binario(1,:));
        display('Error de clasificacion 0 Clasificador R1vsAll')
    else
        Error_Clasif_AdaboostM1(1)=1;
        Clasificador_Aux_R1=fitensemble(X,Y_Binario(1,:),'AdaBoostM1',NLearn_R1,'Tree');
    end
    % Cross Validation
    Clasificador_CV_R1=crossval(Clasificador_Aux_R1,'KFold',KFold);

    if Muestra_Reducida==1
        
        % Clasificador R2vsR3R4R5
        Y_Training_Clasif_Binario_R2=(Y_Training_Clasif_R2vsR3R4R5==2);
        Clasificador_R2=fitensemble(X_Training_Clasif_R2vsR3R4R5,Y_Training_Clasif_Binario_R2,'AdaBoostM1',NLearn_R2,'Tree');  
        if isempty(Clasificador_R2.TrainedWeights)
            Clasificador_R2=fitctree(X_Training_Clasif_R2vsR3R4R5,Y_Training_Clasif_Binario_R2);
            Clasificador_Aux_R2=fitctree(X(Y>1),(Y(Y>1))==2);
            display('Error de clasificacion 0 Clasificador R2vsR3R4R5')
        else
            Error_Clasif_AdaboostM1(2)=1;
            Clasificador_Aux_R2=fitensemble(X(Y>1),(Y(Y>1))==2,'AdaBoostM1',NLearn_R2,'Tree');
        end
        % Cross Validation
        Clasificador_CV_R2=crossval(Clasificador_Aux_R2,'KFold',KFold);

        % Clasificador R3vsR4R5
        Y_Training_Clasif_Binario_R3=(Y_Training_Clasif_R3vsR4R5==3);
        Clasificador_R3=fitensemble(X_Training_Clasif_R3vsR4R5,Y_Training_Clasif_Binario_R3,'AdaBoostM1',NLearn_R3,'Tree');
        if isempty(Clasificador_R3.TrainedWeights)
            Clasificador_R3=fitctree(X_Training_Clasif_R3vsR4R5,Y_Training_Clasif_Binario_R3);
            Clasificador_Aux_R3=fitctree(X(Y>2),(Y(Y>2))==3);
            display('Error de clasificacion 0 Clasificador R3vsR4R5')
        else      
            Error_Clasif_AdaboostM1(3)=1;
            Clasificador_Aux_R3=fitensemble(X(Y>2),(Y(Y>2))==3,'AdaBoostM1',NLearn_R3,'Tree');
        end
        % Cross Validation
        Clasificador_CV_R3=crossval(Clasificador_Aux_R3,'KFold',KFold);
    
        % Clasificador R4vsR5
        Y_Training_Clasif_Binario_R4=(Y_Training_Clasif_R4vsR5==4);
        Clasificador_R4=fitensemble(X_Training_Clasif_R4vsR5,Y_Training_Clasif_Binario_R4,'AdaBoostM1',NLearn_R4,'Tree');   
        if isempty(Clasificador_R4.TrainedWeights)
            Clasificador_R4=fitctree(X_Training_Clasif_R4vsR5,Y_Training_Clasif_Binario_R4);
            Clasificador_Aux_R4=fitctree(X(Y>3),(Y(Y>3))==4);
            display('Error de clasificacion 0 Clasificador R4vsR5')
        else
            Error_Clasif_AdaboostM1(4)=1;
            Clasificador_Aux_R4=fitensemble(X(Y>3),(Y(Y>3))==4,'AdaBoostM1',NLearn_R4,'Tree');
        end
        % Cross Validation
        Clasificador_CV_R4=crossval(Clasificador_Aux_R4,'KFold',KFold);   
    else
        % Clasificador R2vsR3R4R5
        Y_Training_Clasif_Binario_R2=(Y_Training==2);
        Clasificador_R2=fitensemble(X_Training,Y_Training_Clasif_Binario_R2,'AdaBoostM1',NLearn_R2,'Tree');  
        if isempty(Clasificador_R2.TrainedWeights)
            Clasificador_R2=fitctree(X_Training,Y_Training_Clasif_Binario_R2);
            Clasificador_Aux_R2=fitctree(X,Y_Binario(2,:));
            display('Error de clasificacion 0 Clasificador R2vsR3R4R5')
        else
            Error_Clasif_AdaboostM1(2)=1;
            Clasificador_Aux_R2=fitensemble(X,Y_Binario(2,:),'AdaBoostM1',NLearn_R2,'Tree');
        end
        % Cross Validation
        Clasificador_CV_R2=crossval(Clasificador_Aux_R2,'KFold',KFold);
     
        % Clasificador R3vsR4R5
        Y_Training_Clasif_Binario_R3=(Y_Training==3);
        Clasificador_R3=fitensemble(X_Training,Y_Training_Clasif_Binario_R3,'AdaBoostM1',NLearn_R3,'Tree');
        if isempty(Clasificador_R3.TrainedWeights)
            Clasificador_R3=fitctree(X_Training,Y_Training_Clasif_Binario_R3);
            Clasificador_Aux_R3=fitctree(X,Y_Binario(3,:));
            display('Error de clasificacion 0 Clasificador R3vsR4R5')
        else      
            Error_Clasif_AdaboostM1(3)=1;
            Clasificador_Aux_R3=fitensemble(X,Y_Binario(3,:),'AdaBoostM1',NLearn_R3,'Tree');
        end
        % Cross Validation
        Clasificador_CV_R3=crossval(Clasificador_Aux_R3,'KFold',KFold);

        % Clasificador R4vsR5
        Y_Training_Clasif_Binario_R4=(Y_Training==4);
        Clasificador_R4=fitensemble(X_Training,Y_Training_Clasif_Binario_R4,'AdaBoostM1',NLearn_R4,'Tree');   
        if isempty(Clasificador_R4.TrainedWeights)
            Clasificador_R4=fitctree(X_Training,Y_Training_Clasif_Binario_R4);
            Clasificador_Aux_R4=fitctree(X,Y_Binario(4,:));
            display('Error de clasificacion 0 Clasificador R4vsR5')
        else
            Error_Clasif_AdaboostM1(4)=1;
            Clasificador_Aux_R4=fitensemble(X,Y_Binario(4,:),'AdaBoostM1',NLearn_R4,'Tree');
        end
        % Cross Validation
        Clasificador_CV_R4=crossval(Clasificador_Aux_R4,'KFold',KFold);
    end

    % Cascada de clasificadores. Validacion mediante particion de prueba 
    Clasificacion=zeros([Long_X_Test,1]);

    for i=1:Long_X_Test      
        Clasificacion(i)=predict(Clasificador_R1,X_Test(i,:));
        if Clasificacion(i)==1
        else
            Clasificacion(i)=predict(Clasificador_R2,X_Test(i,:));
            if Clasificacion(i)==1
                Clasificacion(i)=2;
            else
                Clasificacion(i)=predict(Clasificador_R3,X_Test(i,:));
                if Clasificacion(i)==1
                    Clasificacion(i)=3;
                else
                    Clasificacion(i)=predict(Clasificador_R4,X_Test(i,:));
                    if Clasificacion(i)==1
                        Clasificacion(i)=4;
                    else
                        Clasificacion(i)=5;
                    end
                end
            end
        end
    end

elseif Metodo==2    
% Clasificador Adaboost Multiclase (todas las clases)

    % Entrenamiento del clasificador
    Clasificador_AdaboostM2=fitensemble(X_Training,Y_Training,'AdaBoostM2',NLearn_Multi,'Tree');
    % Cross Validation
    Clasificador_AdaboostM2_Aux=fitensemble(X,Y,'AdaBoostM2',NLearn_Multi,'Tree');
    Clasificador_AdaboostM2_CV=crossval(Clasificador_AdaboostM2_Aux,'KFold',KFold);
    
    % validacion mediante particion de prueba 
    Clasificacion=predict(Clasificador_AdaboostM2,X_Test);
    
elseif Metodo==3  
% Aplicacion de clasificador Adaboost Biclase para R1vsAll, y Adaboost 
% Multiclase para el resto de clases

    % Clasificador R1vsAll
    Y_Training_Clasif_Binario_R1=(Y_Training==1);
    Clasificador_R1=fitensemble(X_Training,Y_Training_Clasif_Binario_R1,'AdaBoostM1',NLearn_R1,'Tree');
    if isempty(Clasificador_R1.TrainedWeights)
        Clasificador_R1=fitctree(X_Training,Y_Training_Clasif_Binario_R1);
        Clasificador_Aux_R1=fitctree(X,Y_Binario(1,:));
        display('Error de clasificacion 0 Clasificador R1vsAll')
    else
        Error_Clasif_AdaboostM1(1)=1;
        Clasificador_Aux_R1=fitensemble(X,Y_Binario(1,:),'AdaBoostM1',NLearn_R1,'Tree');
    end
    % Cross Validation
    Clasificador_CV_R1=crossval(Clasificador_Aux_R1,'KFold',KFold);
    
    % Clasificador Multiclase
    if Muestra_Reducida==1
        Clasificador_AdaboostM2=fitensemble(X_Training_Clasif_R2vsR3R4R5,Y_Training_Clasif_R2vsR3R4R5,'AdaBoostM2',NLearn_Multi,'Tree');
        % Cross Validation
        Clasificador_AdaboostM2_Aux=fitensemble(X(Y>1,:),Y(Y>1,:),'AdaBoostM2',NLearn_Multi,'Tree');
        Clasificador_AdaboostM2_CV=crossval(Clasificador_AdaboostM2_Aux,'KFold',KFold);
    else
        Clasificador_AdaboostM2=fitensemble(X_Training,Y_Training,'AdaBoostM2',NLearn_Multi,'Tree');
        % Cross Validation
        Clasificador_AdaboostM2_Aux=fitensemble(X,Y,'AdaBoostM2',NLearn_Multi,'Tree');
        Clasificador_AdaboostM2_CV=crossval(Clasificador_AdaboostM2_Aux,'KFold',KFold);
    end
    
    % Cascada de clasificadores. Validacion mediante particion de prueba 
    Clasificacion=zeros([Long_X_Test,1]);

    for i=1:Long_X_Test      
        Clasificacion(i)=predict(Clasificador_R1,X_Test(i,:));
        if Clasificacion(i)==1
        else
            Clasificacion(i)=predict(Clasificador_AdaboostM2,X_Test(i,:));
        end
    end

elseif Metodo==4
% Aplicacion de clasificador Adaboost Biclase para R1vsAll y R2vsR3R4R5 y 
% Adaboost Multiclase para el resto de clases

    % Clasificador R1vsAll
    Y_Training_Clasif_Binario_R1=(Y_Training==1);
    Clasificador_R1=fitensemble(X_Training,Y_Training_Clasif_Binario_R1,'AdaBoostM1',NLearn_R1,'Tree');
    if isempty(Clasificador_R1.TrainedWeights)
        Clasificador_R1=fitctree(X_Training,Y_Training_Clasif_Binario_R1);
        Clasificador_Aux_R1=fitctree(X,Y_Binario(1,:));
        display('Error de clasificacion 0 Clasificador R1vsAll')
    else
        Error_Clasif_AdaboostM1(1)=1;
        Clasificador_Aux_R1=fitensemble(X,Y_Binario(1,:),'AdaBoostM1',NLearn_R1,'Tree');
    end
    % Cross Validation
    Clasificador_CV_R1=crossval(Clasificador_Aux_R1,'KFold',KFold);
    
    if Muestra_Reducida==1  
        % Clasificador R2vsR3R4R5
        Y_Training_Clasif_Binario_R2=(Y_Training_Clasif_R2vsR3R4R5==2);
        Clasificador_R2=fitensemble(X_Training_Clasif_R2vsR3R4R5,Y_Training_Clasif_Binario_R2,'AdaBoostM1',NLearn_R2,'Tree');  
        if isempty(Clasificador_R2.TrainedWeights)
            Clasificador_R2=fitctree(X_Training_Clasif_R2vsR3R4R5,Y_Training_Clasif_Binario_R2);
            Clasificador_Aux_R2=fitctree(X(Y>1),(Y(Y>1))==2);
            display('Error de clasificacion 0 Clasificador R2vsR3R4R5')
        else
            Error_Clasif_AdaboostM1(2)=1;
            Clasificador_Aux_R2=fitensemble(X(Y>1),(Y(Y>1))==2,'AdaBoostM1',NLearn_R2,'Tree');
        end
        % Cross Validation
        Clasificador_CV_R2=crossval(Clasificador_Aux_R2,'KFold',KFold);

        Clasificador_AdaboostM2=fitensemble(X_Training_Clasif_R2vsR3R4R5,Y_Training_Clasif_R2vsR3R4R5,'AdaBoostM2',NLearn_Multi,'Tree');
        % Cross Validation
        Clasificador_AdaboostM2_Aux=fitensemble(X(Y>1,:),Y(Y>1,:),'AdaBoostM2',NLearn_Multi,'Tree');
        Clasificador_AdaboostM2_CV=crossval(Clasificador_AdaboostM2_Aux,'KFold',KFold);
    else
        % Clasificador R2vsR3R4R5
        Y_Training_Clasif_Binario_R2=(Y_Training==2);
        Clasificador_R2=fitensemble(X_Training,Y_Training_Clasif_Binario_R2,'AdaBoostM1',NLearn_R2,'Tree');  
        if isempty(Clasificador_R2.TrainedWeights)
            Clasificador_R2=fitctree(X_Training,Y_Training_Clasif_Binario_R2);
            Clasificador_Aux_R2=fitctree(X,Y_Binario(2,:));
            display('Error de clasificacion 0 Clasificador R2vsR3R4R5')
        else
            Error_Clasif_AdaboostM1(2)=1;
            Clasificador_Aux_R2=fitensemble(X,Y_Binario(2,:),'AdaBoostM1',NLearn_R2,'Tree');
        end
        % Cross Validation
        Clasificador_CV_R2=crossval(Clasificador_Aux_R2,'KFold',KFold);
        
        Clasificador_AdaboostM2=fitensemble(X_Training,Y_Training,'AdaBoostM2',NLearn_Multi,'Tree');
        % Cross Validation
        Clasificador_AdaboostM2_Aux=fitensemble(X,Y,'AdaBoostM2',NLearn_Multi,'Tree');
        Clasificador_AdaboostM2_CV=crossval(Clasificador_AdaboostM2_Aux,'KFold',KFold);
        
    end
    
    % Cascada de clasificadores. Validacion mediante particion de prueba 
    Clasificacion=zeros([Long_X_Test,1]);

    for i=1:Long_X_Test      
        Clasificacion(i)=predict(Clasificador_R1,X_Test(i,:));
        if Clasificacion(i)==1
        else
            Clasificacion(i)=predict(Clasificador_R2,X_Test(i,:));
            if Clasificacion(i)==1
                Clasificacion(i)=2;
            else
                Clasificacion(i)=predict(Clasificador_AdaboostM2,X_Test(i,:));
            end
        end
    end

elseif Metodo==5||Metodo==6
% Metodo 5: Aplicacion de todos los clasificadores binarios, multiclase
% para desempatar resultados

% Metodo 6: Aplicacion de todos los clasificadores binarios y multiclase,
% primando redultados correspondientes a la clase 1

    % Clasificador R1vsAll
    Y_Training_Clasif_Binario_R1=(Y_Training==1);
    Clasificador_R1=fitensemble(X_Training,Y_Training_Clasif_Binario_R1,'AdaBoostM1',NLearn_R1,'Tree');

    if isempty(Clasificador_R1.TrainedWeights)
        Clasificador_R1=fitctree(X_Training,Y_Training_Clasif_Binario_R1);
        Clasificador_Aux_R1=fitctree(X,Y_Binario(1,:));
        display('Error de clasificacion 0 Clasificador R1vsAll')
    else
        Error_Clasif_AdaboostM1(1)=1;
        Clasificador_Aux_R1=fitensemble(X,Y_Binario(1,:),'AdaBoostM1',NLearn_R1,'Tree');
    end
    % Cross Validation
    Clasificador_CV_R1=crossval(Clasificador_Aux_R1,'KFold',KFold);
   
    % Clasificador R2vsAll
    Y_Training_Clasif_Binario_R2=(Y_Training==2);
    Clasificador_R2=fitensemble(X_Training,Y_Training_Clasif_Binario_R2,'AdaBoostM1',NLearn_R2,'Tree');
    
    if isempty(Clasificador_R2.TrainedWeights)
        Clasificador_R2=fitctree(X_Training,Y_Training_Clasif_Binario_R2);
        Clasificador_Aux_R2=fitctree(X,Y_Binario(2,:));
        display('Error de clasificacion 0 Clasificador R2vsAll')
    else
        Error_Clasif_AdaboostM1(2)=1;
        Clasificador_Aux_R2=fitensemble(X,Y_Binario(2,:),'AdaBoostM1',NLearn_R2,'Tree');
    end
    % Cross Validation
    Clasificador_CV_R2=crossval(Clasificador_Aux_R2,'KFold',KFold);
   
    % Clasificador R3vsAll
    Y_Training_Clasif_Binario_R3=(Y_Training==3);
    Clasificador_R3=fitensemble(X_Training,Y_Training_Clasif_Binario_R3,'AdaBoostM1',NLearn_R3,'Tree');
    
    if isempty(Clasificador_R3.TrainedWeights)
        Clasificador_R3=fitctree(X_Training,Y_Training_Clasif_Binario_R3);
        Clasificador_Aux_R3=fitctree(X,Y_Binario(3,:));
        display('Error de clasificacion 0 Clasificador R3vsAll')
    else      
        Error_Clasif_AdaboostM1(3)=1;
        Clasificador_Aux_R3=fitensemble(X,Y_Binario(3,:),'AdaBoostM1',NLearn_R3,'Tree');
    end
    % Cross Validation
    Clasificador_CV_R3=crossval(Clasificador_Aux_R3,'KFold',KFold);
  
    % Clasificador R4vsAll
    Y_Training_Clasif_Binario_R4=(Y_Training==4);
    Clasificador_R4=fitensemble(X_Training,Y_Training_Clasif_Binario_R4,'AdaBoostM1',NLearn_R4,'Tree');   
    
    if isempty(Clasificador_R4.TrainedWeights)
        Clasificador_R4=fitctree(X_Training,Y_Training_Clasif_Binario_R4);
        Clasificador_Aux_R4=fitctree(X,Y_Binario(4,:));
        display('Error de clasificacion 0 Clasificador R4vsAll')
    else
        Error_Clasif_AdaboostM1(4)=1;
        Clasificador_Aux_R4=fitensemble(X,Y_Binario(4,:),'AdaBoostM1',NLearn_R4,'Tree');
    end
    % Cross Validation
    Clasificador_CV_R4=crossval(Clasificador_Aux_R4,'KFold',KFold);

    % Clasificador R5vsAll
    Y_Training_Clasif_Binario_R5=(Y_Training==5);
    Clasificador_R5=fitensemble(X_Training,Y_Training_Clasif_Binario_R5,'AdaBoostM1',NLearn_R5,'Tree');   
    
    if isempty(Clasificador_R5.TrainedWeights)
        Clasificador_R5=fitctree(X_Training,Y_Training_Clasif_Binario_R5);
        Clasificador_Aux_R5=fitctree(X,Y_Binario(5,:));
        display('Error de clasificacion 0 Clasificador R5vsAll')
    else
        Error_Clasif_AdaboostM1(5)=1;
        Clasificador_Aux_R5=fitensemble(X,Y_Binario(5,:),'AdaBoostM1',NLearn_R5,'Tree');
    end
    % Cross Validation
    Clasificador_CV_R5=crossval(Clasificador_Aux_R5,'KFold',KFold);
    
    % Clasificador Multiclase
    Clasificador_AdaboostM2=fitensemble(X_Training,Y_Training,'AdaBoostM2',NLearn_Multi,'Tree');
    % Cross Validation
    Clasificador_AdaboostM2_Aux=fitensemble(X,Y,'AdaBoostM2',NLearn_Multi,'Tree');
    Clasificador_AdaboostM2_CV=crossval(Clasificador_AdaboostM2_Aux,'KFold',KFold);
   
    
    Clasificacion_Tot=zeros([6,Long_X_Test]);
    
    Clasificacion_Tot(1,:)=predict(Clasificador_R1,X_Test);
    Clasificacion_Tot(2,:)=predict(Clasificador_R2,X_Test);
    Clasificacion_Tot(3,:)=predict(Clasificador_R3,X_Test);
    Clasificacion_Tot(4,:)=predict(Clasificador_R4,X_Test);
    Clasificacion_Tot(5,:)=predict(Clasificador_R5,X_Test);
    
    Clasificacion=zeros([Long_X_Test,1]);

    if Metodo==5
        % Validacion mediante particion de prueba 
        for i=1:Long_X_Test
            if sum(Clasificacion_Tot(:,i))==1
                for k=1:Num_Clases
                    if (Clasificacion_Tot(k,i))==1
                        Clasificacion(i)=k;
                    else
                    end
                end
            else
                Clasificacion_Tot(6,i)=predict(Clasificador_AdaboostM2,X_Test(i,:));
                Clasificacion(i)=Clasificacion_Tot(6,i);
            end
        end
    else
        Clasificacion_Tot(6,:)=predict(Clasificador_AdaboostM2,X_Test);

        % Validacion mediante particion de prueba 
        for i=1:Long_X_Test
            if (Clasificacion_Tot(1,i)==0)||(Clasificacion_Tot(6,i)~=1)
                if (sum(Clasificacion_Tot([2 3 4 5],i)>0))==0
                    if (Clasificacion_Tot(6,i)~=1)
                        Clasificacion(i)=Clasificacion_Tot(6,i);
                    else
                        Clasificacion(i)=4;
                    end
                elseif (sum(Clasificacion_Tot([2 3 4 5],i)>0))>1
                    Clasificacion(i)=Clasificacion_Tot(6,i);
                else
                    Clasificacion(i)=1+find(Clasificacion_Tot([2 3 4 5],i)>0);
                end
            else
                Clasificacion(i)=1;
            end
        end      
    end
    
else
    display('Introduzca un valor para seleccionar el mtodo de clasificacin comprendido entre 0 y 4')
    Error_Clasif_AdaboostM1=9999; % Valor de control, por si el usuario ha elegido un mtodo inexistente
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%     ERROR DE RESISTITUCION Y ERROR DE CLASIFICACION MEDIANTE CV
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if Metodo==1
    if Error_Clasif_AdaboostM1(1)==1
        RsLoss_R1=resubLoss(Clasificador_R1,'Mode','Cumulative');
        GenError_R1=kfoldLoss(Clasificador_CV_R1,'Mode','Cumulative');
    else
        RsLoss_R1=resubLoss(Clasificador_R1)*ones(1,NLearn_R1)';
        GenError_R1=kfoldLoss(Clasificador_CV_R3)*ones(1,NLearn_R1)';
    end
    RsLoss_R1=RsLoss_R1*100;
    GenError_R1=GenError_R1*100;
    
    if Error_Clasif_AdaboostM1(2)==1
        RsLoss_R2=resubLoss(Clasificador_R2,'Mode','Cumulative');
        GenError_R2=kfoldLoss(Clasificador_CV_R2,'Mode','Cumulative');
    else
        RsLoss_R2=resubLoss(Clasificador_R2)*ones(1,NLearn_R2)';
        GenError_R2=kfoldLoss(Clasificador_CV_R3)*ones(1,NLearn_R2)';
    end
    RsLoss_R2=RsLoss_R2*100;
    GenError_R2=GenError_R2*100;

    if Error_Clasif_AdaboostM1(3)==1
        RsLoss_R3=resubLoss(Clasificador_R3,'Mode','Cumulative');
        GenError_R3=kfoldLoss(Clasificador_CV_R3,'Mode','Cumulative');
    else
        RsLoss_R3=resubLoss(Clasificador_R3)*ones(1,NLearn_R3)';
        GenError_R3=kfoldLoss(Clasificador_CV_R3)*ones(1,NLearn_R3)';
    end
    RsLoss_R3=RsLoss_R3*100;
    GenError_R3=GenError_R3*100;

    if Error_Clasif_AdaboostM1(4)==1
        RsLoss_R4=resubLoss(Clasificador_R4,'Mode','Cumulative');
        GenError_R4=kfoldLoss(Clasificador_CV_R4,'Mode','Cumulative');
    else
        RsLoss_R4=resubLoss(Clasificador_R4)*ones(1,NLearn_R4)';
        GenError_R4=kfoldLoss(Clasificador_CV_R4)*ones(1,NLearn_R4)';
    end
    RsLoss_R4=RsLoss_R4*100;
    GenError_R4=GenError_R4*100;

    figure;
    subplot(2,2,1);  
    plot((1:1:NLearn_R1),RsLoss_R1,(1:1:NLearn_R1),GenError_R1);
    title('\fontsize{14}Clasificador R1vsAll');
    legend('\fontsize{13}Resubstitution Classification Cost','\fontsize{13}Cross-Validation Loss');
    xlabel('\fontsize{13}Number of Learning Cycles');ylabel('\fontsize{13}Loss (%)');
    ylim([0 inf]);
    subplot(2,2,2);  
    plot((1:1:NLearn_R2),RsLoss_R2,(1:1:NLearn_R2),GenError_R2);
    title('\fontsize{14}Clasificador R2vsR3R4R5');
    legend('\fontsize{13}Resubstitution Classification Cost','\fontsize{13}Cross-Validation Loss');
    xlabel('\fontsize{13}Number of Learning Cycles');ylabel('\fontsize{13}Loss (%)');
    ylim([0 inf]);
    subplot(2,2,3);  
    plot((1:1:NLearn_R3),RsLoss_R3,(1:1:NLearn_R3),GenError_R3);
    title('\fontsize{14}Clasificador R3vsR4R5');
    legend('\fontsize{13}Resubstitution Classification Cost','\fontsize{13}Cross-Validation Loss');
    xlabel('\fontsize{13}Number of Learning Cycles');ylabel('\fontsize{13}Loss (%)');
    ylim([0 inf]);
    subplot(2,2,4);  
    plot((1:1:NLearn_R4),RsLoss_R4,(1:1:NLearn_R4),GenError_R4);
    title('\fontsize{14}Clasificador R4vsR5');
    legend('\fontsize{13}Resubstitution Classification Cost','\fontsize{13}Cross-Validation Loss');
    xlabel('\fontsize{13}Number of Learning Cycles');ylabel('\fontsize{13}Loss (%)');
    ylim([0 inf]);
    
elseif Metodo==2
    RsLoss_Multiclase=100*resubLoss(Clasificador_AdaboostM2,'Mode','Cumulative');
    GenError_Multiclase=100*kfoldLoss(Clasificador_AdaboostM2_CV,'Mode','Cumulative');

    plot((1:1:NLearn_Multi),RsLoss_Multiclase,(1:1:NLearn_Multi),GenError_Multiclase);
    title('\fontsize{24}Clasificador Multiclase');
    legend('\fontsize{18}Resubstitution Classification Cost','\fontsize{18}Cross-Validation Loss');
    xlabel('\fontsize{20}Number of Learning Cycles');ylabel('\fontsize{20}Loss (%)');
    ylim([0 inf]);
    
elseif Metodo==3
    if Error_Clasif_AdaboostM1(1)==1
        RsLoss_R1=resubLoss(Clasificador_R1,'Mode','Cumulative');
        GenError_R1=kfoldLoss(Clasificador_CV_R1,'Mode','Cumulative');
    else
        RsLoss_R1=resubLoss(Clasificador_R1)*ones(1,NLearn_R1)';
        GenError_R1=kfoldLoss(Clasificador_CV_R1)*ones(1,NLearn_R1)';
    end
    RsLoss_R1=RsLoss_R1*100;
    GenError_R1=GenError_R1*100;
    
    RsLoss_Multiclase=100*resubLoss(Clasificador_AdaboostM2,'Mode','Cumulative');
    GenError_Multiclase=100*kfoldLoss(Clasificador_AdaboostM2_CV,'Mode','Cumulative');
  
    figure;
    subplot(2,1,1);  
    plot((1:1:NLearn_R1),RsLoss_R1,(1:1:NLearn_R1),GenError_R1);
    title('\fontsize{22}Clasificador R1vsAll');
    legend('\fontsize{12}Resubstitution Classification Cost','\fontsize{12}Cross-Validation Loss');
    xlabel('\fontsize{20}Number of Learning Cycles');ylabel('\fontsize{20}Loss (%)');
    ylim([0 inf]);
    subplot(2,1,2);  
    plot((1:1:NLearn_Multi),RsLoss_Multiclase,(1:1:NLearn_Multi),GenError_Multiclase);
    title('\fontsize{22}Clasificador Multiclase');
    legend('\fontsize{12}Resubstitution Classification Cost','\fontsize{12}Cross-Validation Loss');
    xlabel('\fontsize{20}Number of Learning Cycles');ylabel('\fontsize{20}Loss (%)');
    ylim([0 inf]);

elseif Metodo==4
    if Error_Clasif_AdaboostM1(1)==1
        RsLoss_R1=resubLoss(Clasificador_R1,'Mode','Cumulative');
        GenError_R1=kfoldLoss(Clasificador_CV_R1,'Mode','Cumulative');
    else
        RsLoss_R1=resubLoss(Clasificador_R1)*ones(1,NLearn_R1)';
        GenError_R1=kfoldLoss(Clasificador_CV_R1)*ones(1,NLearn_R1)';
    end
    RsLoss_R1=RsLoss_R1*100;
    GenError_R1=GenError_R1*100;
    
    if Error_Clasif_AdaboostM1(2)==1
        RsLoss_R2=resubLoss(Clasificador_R2,'Mode','Cumulative');
        GenError_R2=kfoldLoss(Clasificador_CV_R2,'Mode','Cumulative');
    else
        RsLoss_R2=resubLoss(Clasificador_R2)*ones(1,NLearn_R2)';
        GenError_R2=kfoldLoss(Clasificador_CV_R2)*ones(1,NLearn_R2)';
    end
    RsLoss_R2=RsLoss_R2*100;
    GenError_R2=GenError_R2*100;

    RsLoss_Multiclase=100*resubLoss(Clasificador_AdaboostM2,'Mode','Cumulative');
    GenError_Multiclase=100*kfoldLoss(Clasificador_AdaboostM2_CV,'Mode','Cumulative');
    
    figure;
    subplot(2,2,1);
    plot((1:1:NLearn_R1),RsLoss_R1,(1:1:NLearn_R1),GenError_R1);
    title('\fontsize{14}Clasificador R1vsAll');
    legend('\fontsize{13}Resubstitution Classification Cost','\fontsize{13}Cross-Validation Loss');
    xlabel('\fontsize{13}Number of Learning Cycles');ylabel('\fontsize{13}Loss (%)');
    ylim([0 inf]);
    subplot(2,2,2);
    plot((1:1:NLearn_R2),RsLoss_R2,(1:1:NLearn_R2),GenError_R2);
    title('\fontsize{14}Clasificador R2vsR3R4R5');
    legend('\fontsize{13}Resubstitution Classification Cost','\fontsize{13}Cross-Validation Loss');
    xlabel('\fontsize{13}Number of Learning Cycles');ylabel('\fontsize{13}Loss (%)');
    ylim([0 inf]);
    subplot(2,2,[3,4]); 
    plot((1:1:NLearn_Multi),RsLoss_Multiclase,(1:1:NLearn_Multi),GenError_Multiclase);
    title('\fontsize{14}Clasificador Multiclase');
    legend('\fontsize{13}Resubstitution Classification Cost','\fontsize{13}Cross-Validation Loss');
    xlabel('\fontsize{13}Number of Learning Cycles');ylabel('\fontsize{13}Loss (%)');
    ylim([0 inf]);

elseif Metodo==5||Metodo==6
    if Error_Clasif_AdaboostM1(1)==1
        RsLoss_R1=resubLoss(Clasificador_R1,'Mode','Cumulative');
        GenError_R1=kfoldLoss(Clasificador_CV_R1,'Mode','Cumulative');
    else
        RsLoss_R1=resubLoss(Clasificador_R1)*ones(1,NLearn_R1)';
        GenError_R1=kfoldLoss(Clasificador_CV_R1)*ones(1,NLearn_R1)';
    end
    RsLoss_R1=RsLoss_R1*100;
    GenError_R1=GenError_R1*100;
    
    if Error_Clasif_AdaboostM1(2)==1
        RsLoss_R2=resubLoss(Clasificador_R2,'Mode','Cumulative');
        GenError_R2=kfoldLoss(Clasificador_CV_R2,'Mode','Cumulative');
    else
        RsLoss_R2=resubLoss(Clasificador_R2)*ones(1,NLearn_R2)';
        GenError_R2=kfoldLoss(Clasificador_CV_R2)*ones(1,NLearn_R2)';
    end
    RsLoss_R2=RsLoss_R2*100;
    GenError_R2=GenError_R2*100;

    if Error_Clasif_AdaboostM1(3)==1
        RsLoss_R3=resubLoss(Clasificador_R3,'Mode','Cumulative');
        GenError_R3=kfoldLoss(Clasificador_CV_R3,'Mode','Cumulative');
    else
        RsLoss_R3=resubLoss(Clasificador_R3)*ones(1,NLearn_R3)';
        GenError_R3=kfoldLoss(Clasificador_CV_R3)*ones(1,NLearn_R3)';
    end
    RsLoss_R3=RsLoss_R3*100;
    GenError_R3=GenError_R3*100;

    if Error_Clasif_AdaboostM1(4)==1
        RsLoss_R4=resubLoss(Clasificador_R4,'Mode','Cumulative');
        GenError_R4=kfoldLoss(Clasificador_CV_R4,'Mode','Cumulative');
    else
        RsLoss_R4=resubLoss(Clasificador_R4)*ones(1,NLearn_R4)';
        GenError_R4=kfoldLoss(Clasificador_CV_R4)*ones(1,NLearn_R4)';
    end
    RsLoss_R4=RsLoss_R4*100;
    GenError_R4=GenError_R4*100;

    if Error_Clasif_AdaboostM1(5)==1
        RsLoss_R5=resubLoss(Clasificador_R5,'Mode','Cumulative');
        GenError_R5=kfoldLoss(Clasificador_CV_R5,'Mode','Cumulative');
    else
        RsLoss_R5=resubLoss(Clasificador_R5)*ones(1,NLearn_R5)';
        GenError_R5=kfoldLoss(Clasificador_CV_R5)*ones(1,NLearn_R5)';
    end
    RsLoss_R5=RsLoss_R5*100;
    GenError_R5=GenError_R5*100;
    
    RsLoss_Multiclase=100*resubLoss(Clasificador_AdaboostM2,'Mode','Cumulative');
    GenError_Multiclase=100*kfoldLoss(Clasificador_AdaboostM2_CV,'Mode','Cumulative');
    
    figure;
    subplot(3,2,1);  
    plot((1:1:NLearn_R1),RsLoss_R1,(1:1:NLearn_R1),GenError_R1);
    title('\fontsize{14}Clasificador R1vsAll');
    legend('\fontsize{13}Resubstitution Classification Cost','\fontsize{13}Cross-Validation Loss');
    xlabel('\fontsize{13}Number of Learning Cycles');ylabel('\fontsize{13}Loss (%)');
    ylim([0 inf]);
    subplot(3,2,2);  
    plot((1:1:NLearn_R2),RsLoss_R2,(1:1:NLearn_R2),GenError_R2);
    title('\fontsize{14}Clasificador R2vsAll');
    legend('\fontsize{13}Resubstitution Classification Cost','\fontsize{13}Cross-Validation Loss');
    xlabel('\fontsize{13}Number of Learning Cycles');ylabel('\fontsize{13}Loss (%)');
    ylim([0 inf]);
    subplot(3,2,3);  
    plot((1:1:NLearn_R3),RsLoss_R3,(1:1:NLearn_R3),GenError_R3);
    title('\fontsize{14}Clasificador R3vsAll');
    legend('\fontsize{13}Resubstitution Classification Cost','\fontsize{13}Cross-Validation Loss');
    xlabel('\fontsize{13}Number of Learning Cycles');ylabel('\fontsize{13}Loss (%)');
    ylim([0 inf]);
    subplot(3,2,4);  
    plot((1:1:NLearn_R4),RsLoss_R4,(1:1:NLearn_R4),GenError_R4);
    title('\fontsize{14}Clasificador R4vsAll');
    legend('\fontsize{13}Resubstitution Classification Cost','\fontsize{13}Cross-Validation Loss');
    xlabel('\fontsize{13}Number of Learning Cycles');ylabel('\fontsize{13}Loss (%)');
    ylim([0 inf]);
    subplot(3,2,5);  
    plot((1:1:NLearn_R5),RsLoss_R5,(1:1:NLearn_R5),GenError_R5);
    title('\fontsize{14}Clasificador R4vsAll');
    legend('\fontsize{13}Resubstitution Classification Cost','\fontsize{13}Cross-Validation Loss');
    xlabel('\fontsize{13}Number of Learning Cycles');ylabel('\fontsize{13}Loss (%)');
    ylim([0 inf]);
    subplot(3,2,6); 
    plot((1:1:NLearn_Multi),RsLoss_Multiclase,(1:1:NLearn_Multi),GenError_Multiclase);
    title('\fontsize{14}Clasificador Multiclase');
    legend('\fontsize{13}Resubstitution Classification Cost','\fontsize{13}Cross-Validation Loss');
    xlabel('\fontsize{13}Number of Learning Cycles');ylabel('\fontsize{13}Loss (%)');
    ylim([0 inf]);
else
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%        
%    MATRIZ DE CONFUSION Y CURVA CARACTERISTICA OPERATIVA DEL RECEPTOR
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if Error_Clasif_AdaboostM1==9999
else
    [Confusion_Matrix,order]=confusionmat(Y_Test,Clasificacion);

    % Preparamos los datos test para representar la matriz de confusion
    Y_Clasificacion_Mat_Conf=zeros(Num_Clases,Long_X_Test);
    for i=1:Num_Clases
        Y_Clasificacion_Mat_Conf(i,(Clasificacion==i))=1;
    end

    % Representamos la matriz de confusion
    figure
    plotconfusion(Y_Test_Mat_Conf,Y_Clasificacion_Mat_Conf)

    % Representamos la curva caracteristica operativa del receptor
    figure
    plotroc(Y_Test_Mat_Conf,Y_Clasificacion_Mat_Conf)
end